package com.autentia.tutoriales.axis2.cliente;

import java.net.URL;
import javax.xml.namespace.QName;
import org.apache.axiom.om.*;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.client.async.AxisCallback;
import org.apache.axis2.context.ConfigurationContext;

/**
 * Ejemplo del uso el API de cliente de Axis2 para conseguir invocar un método
 * con distintos patrones de intercambio de mensajes (MEP).
 * @author Carlos García. Autentia
 */
public class Axis2MEPApp  {
	
    public static void main(String[] args) {
    	char userOption = '0';
    	
    	try {
    		while ((userOption < '1') || (userOption > '4')){
	    		System.out.println("Ejemplo de invocación a servicios web con distintos Message Exchange Patterns:");
	    		System.out.println("(1) MEP In-Out   Síncrono");
	    		System.out.println("(2) MEP In-Out   Asíncrono");
	    		System.out.println("(3) MEP In-Only. Recepción y procesamiento del mensaje asegurada.");
	    		System.out.println("(4) MEP In-Only. Recepción y procesamiento del mensaje no asegurada.");
	    		System.out.println("Seleccione una opción (1, 2, 3, 4):");
	    		userOption = (char) System.in.read();
    		}
    		
    		// Usamos la configuración por defecto de Axis2
    		ConfigurationContext context	 = null; 															 
        	
        	// URL del WSDL
        	URL	wsdlURL	 = new URL("http://localhost:8080/bibliotecaWS/bibliotecaWS.wsdl");	
        	
        	// Podría ser Null pues sólo hay un servicio en el WSDL
        	QName serviceName = new QName("http://www.adictosaltrabajo.com/spring/ws/schemas", "bibliotecaWSService");						
        	
        	// El WSDL tiene 2 Port elegimos uno.
        	String	portName = "bibliotecaWSSoap12";
        	
        	// Clase cliente básica (No tiene acceso a operaciones más avanzadas como OperationClient)
        	ServiceClient wsClient = new ServiceClient(context, wsdlURL, serviceName, portName);
    		
        	// Operación a invocar    		
	    	QName operation = new QName("http://www.adictosaltrabajo.com/spring/ws/schemas", "BooksInfo");
	    	
	    	// Obtenemos la petición a enviar al servicio web
	    	OMElement	  request   = Axis2MEPApp.getRequest("Servicios Web", "avanzado");    	
    		
	    	
    		switch (userOption){
    			case '1': 
    		    	OMElement  response  = wsClient.sendReceive(operation, request);
    		    	 
    		    	// Imprimimos la respuesta
    		    	System.out.println(response);
    				break;
    			case '2': 
    		    	// CallBack que recibirá la respuesta
    		    	AxisCallback callback = new MyAxisCallback();
    		    	
    		    	wsClient.sendReceiveNonBlocking(operation, request, callback);
    		    	for (int i = 0; i < 5; i++){
    		    		System.out.println("Continuamos trabajando en otras cosas mientras el servidor procesa la petición" +  
    		    						   "(tiempos de red + proceso + configuraciones poco optimas de servidores + ...) y nos responde");
    		    		Thread.sleep(100);
    		    	}
    				break;
    			case '3': 
    		    	wsClient.sendRobust(operation, request);
    				break;
    			case '4': 
    		    	wsClient.fireAndForget(operation, request);
    		    	
    		    	// Esperamos un poco para que le de tiempo a enviar la petición (se envia en background)
    		    	Thread.sleep(200);
    				break;    				
    		}
    	} catch (Exception ex){
    		System.out.println(ex); // Es sólo un tutorial, no me preocupo de gestionar bien las excepciones.
    	}
    	
    	
    	System.out.println("FIN DE LA APLICACION");
    }


    /**
     * @param  categoria Categoría del libro a consultar
     * @param  nivel	 Nivel del libro a consultar, valores permitidos: basico, medio, avanzado
     * @return Devuelve la petición a enviar al servicio web.
     */
	private static OMElement getRequest(String categoria, String nivel){
    	/*
			<BooksInfoRequest  xmlns="http://www.adictosaltrabajo.com/spring/ws/schemas">  
				<categoria>Biología</categoria>
				<nivel>basico</nivel>
			</BooksInfoRequest>
			
 			Nota, también se podría hacer desde un String
 				StAXOMBuilder builder = new StAXOMBuilder(new ByteArrayInputStream("El xml...".getBytes()));
				OMElement	  request = builder.getDocumentElement();
    	 */    	
    	OMFactory factory = OMAbstractFactory.getOMFactory();	
    	OMNamespace namespace  = factory.createOMNamespace("http://www.adictosaltrabajo.com/spring/ws/schemas", "");
    	OMElement request      = factory.createOMElement("BooksInfoRequest", namespace);
    	OMElement categoryNode = factory.createOMElement("categoria",		 namespace);
    	OMElement nivelNode	   = factory.createOMElement("nivel",			 namespace);
    	
    	categoryNode.addChild( factory.createOMText(categoria) );
    	nivelNode.addChild(    factory.createOMText(nivel) );
    	
    	request.addChild(categoryNode);
    	request.addChild(nivelNode);
    	
    	return request;
    }   
}